#!/usr/bin/perl

#┌─────────────────────────────────
#│ ClipBoard v6.21 com-multi : compost.cgi - 2023/07/02
#│ copyright (c) TRANSFORM Since 2012
#│ https://www.transformsite.com/
#└────────────────────────────────

# モジュール宣言
use strict;
use CGI::Carp qw(fatalsToBrowser);
use vars qw(%in %cf);
use lib "./lib";
use CGI::Minimal;

# 設定ファイル認識
require "./init.cgi";
%cf = set_init();

# データ受理
CGI::Minimal::max_read_size($cf{maxdata});
my $cgi = CGI::Minimal->new;
error('容量オーバー') if ($cgi->truncated);
%in = parse_resform($cgi);

# 処理分岐
if ($in{mode} eq 'recv') { recv_data(); }
if ($in{mode} eq 'com_del') { com_del(); }
if ($in{mode} eq 'com_edit') { com_edit(); }

error("不明な処理です");

#-----------------------------------------------------------
#  コメント返信投稿処理
#-----------------------------------------------------------
sub recv_data {
	# 日本語チェック
	if ($cf{jp_word_chk}) {
		if ($in{comment} !~ /(?:[\xC0-\xDF][\x80-\xBF]|[\xE0-\xEF][\x80-\xBF]{2}|[\xF0-\xF7][\x80-\xBF]{3})/) {
			error("本文に日本語が含まれていません");
		}
	}
	
	# 禁止ワードチェック
	my $flg;
	for ( split(/\s+/,$cf{bad_word}) ) {
		if (index($in{comment},$_) >= 0) {
			$flg++;
			last;
		}
	}
	if ($flg) { error('コメントに禁止ワードが含まれているため受付できません!'); }
	
	# ホストチェック
	my ($addr,$host) = get_host();
	my $flg;
	for ( split(/\s+/,$cf{bad_host}) ) {
		if (index($host,$_) >= 0 or index($addr,$_) >= 0) {
			$flg++;
			last;
		}
	}
	if ($flg) { error('コメントは受付できません!!'); }

	# 入力値調整
	if ($in{url} eq 'http://') { $in{url} = ''; }

	# フォーム内容をチェック
	my $err;
	if ($in{name} eq "") { $err .= "名前が入力されていません<br>"; }
	if ($in{comment} eq "") { $err .= "コメントが入力されていません<br>"; }
	if ($err) { error($err); }
	
	# 時間取得
	my $time = time;
	my ($min,$hour,$mday,$mon,$year,$wday) = (localtime($time))[1..6];
	my @wk = ('Sun','Mon','Tue','Wed','Thu','Fri','Sat');
	my $date = sprintf("%04d/%02d/%02d(%s) %02d:%02d",
				$year+1900,$mon+1,$mday,$wk[$wday],$hour,$min);

	# 一時保存
	if ($cf{approve} == 1) {
		save_tmp($date,$host);

	# 直接保存
	} else {
		com_post($date,$host);
	}
	
	# クッキー格納
	set_cookie($in{name},$in{url}) if ($in{cookie});
}

#-------------------------------------------------
#  コメント受理
#-------------------------------------------------
sub com_post {
	my ($date,$host) = @_;

	# 採番
	open(DAT,"+< $cf{resnum}");
	eval "flock(DAT, 2);";
	my $num = <DAT> + 1;
	seek(DAT, 0, 0);
	print DAT $num;
	truncate(DAT, tell(DAT));
	close(DAT);

	# ファイル添付処理
	my ($ext,$w,$h) = res_upload($num) if ($in{upfile});

	# 削除キー暗号化
	my $pwd = encrypt($in{pwd}) if ($in{pwd} ne "");

	# データがないとき
	if (!-e "$cf{reslog}") {
		open(DAT,"> $cf{reslog}");
		close(DAT);

		chmod(0666, "$cf{reslog}");
	}
	open(DAT,"+< $cf{datadir}/log.cgi") or error("open err: $cf{datadir}/log.cgi");
	eval "flock(DAT, 2);";
	my $log = <DAT>;
	close(DAT);
	my ($no,$dat,$nam,$eml,$sub,$com,$url,$hos,$pw,$ex,$col,$color2) = (split(/<>/, $log));

	# コメント返信データオープン
	my $res = 0;
	my ($flg,@data);
	open(DAT,"+< $cf{reslog}");
	eval "flock(DAT, 2);";
	while(<DAT>) {
		my ($num,$ent,$sub,$date,$name,$url,$host,$com,$ex,$w,$h,$chk,$pw) = split(/<>/);

		# 最大コメント数をチェック
		if ($in{no} == $ent) { $res++; }
		if ($res >= $cf{max_res}) {
			$flg = 1;
			last;
		}

		push(@data,$_);
	}

	# 最大コメント数を超えるとき
	my $err;
	if ($flg) {
		close(DAT);

		# 画像削除
		if ($ex) { 
			unlink("$cf{r_upldir}/$no$ex");
		}

		# 連番修正
		open(DAT,"+< $cf{resnum}");
		eval "flock(DAT, 2);";
		my $num = <DAT> - 1;
		seek(DAT, 0, 0);
		print DAT $num;
		truncate(DAT, tell(DAT));
		close(DAT);

		# エラー
		$err .= qq |最大コメント数は$cf{max_res}件のため受理できません<br>\n|;

		if ($err) { error($err); }
	}

	# データ更新
	seek(DAT, 0, 0);
	print DAT "$num<>$in{no}<>$in{sub}<>$date<>$in{name}<>$in{url}<>$host<>$in{comment}<>$ext<>$w<>$h<>0<>$pwd<>\n";
	print DAT @data;
	truncate(DAT, tell(DAT));
	close(DAT);

	# TOPレスカウント更新
	top_rescnt();

	# クッキー格納
	set_cookie($in{name},$in{url}) if ($in{cookie});

	# メール通知
	my $clip = "$num$ext" if ($ext);
	mail_to_res($date,$sub,$clip) if ($cf{res_mailing} == 1);

	# 完了画面
	message("ありがとうございます。コメントを受理しました。");
}

#-----------------------------------------------------------
#  一時ファイル保存
#-----------------------------------------------------------
sub save_tmp {
	my ($date,$host) = @_;
	
	# 採番
	open(NO,"+< $cf{resnum}") or error("open err: $cf{resnum}");
	eval "flock(NO,2);";
	my $num = <NO>;
	
	# 通番ファイル保存
	seek(NO,0,0);
	print NO ++$num;
	truncate(NO,tell(NO));
	close(NO);

	# ファイル添付処理
	my ($ext,$w,$h) = res_upload($num) if ($in{upfile});

	# 削除キー暗号化
	my $pwd = encrypt($in{pwd}) if ($in{pwd} ne "");

	my %res;
	open(DAT,"+< $cf{datadir}/log.cgi") or error("open err: $cf{datadir}/log.cgi");
	eval "flock(DAT, 2);";
	my $log = <DAT>;
	close(DAT);
	my ($no,$dat,$nam,$eml,$sub,$com,$url,$hos,$pw,$ex,$col,$color2) = (split(/<>/, $log));

	# コメント返信データオープン
	my $res = 0;
	my ($flg,@data);
	open(DAT,"+< $cf{reslog}");
	eval "flock(DAT, 2);";
	while(<DAT>) {
		my ($num,$ent,$sub,$date,$name,$url,$host,$com,$ex,$w,$h,$chk,$pw) = split(/<>/);

		# 最大コメント数をチェック
		if ($in{no} == $ent) { $res++; }
		if ($res >= $cf{max_res}) {
			$flg = 1;
			last;
		}

		push(@data,$_);
	}

	# 最大コメント数を超えるとき
	my $err;
	if ($flg) {
		close(DAT);

		# 画像削除
		if ($ex) { 
			unlink("$cf{r_upldir}/$no$ex");
		}

		# 連番修正
		open(DAT,"+< $cf{resnum}");
		eval "flock(DAT, 2);";
		my $num = <DAT> - 1;
		seek(DAT, 0, 0);
		print DAT $num;
		truncate(DAT, tell(DAT));
		close(DAT);

		# エラー
		$err .= qq |最大コメント数は$cf{max_res}件のため受理できません<br>\n|;

		if ($err) { error($err); }
	}

	# ファイル生成
	open(TMP,"+> $cf{logdir}/$num.cgi") or error("write err: $num.cgi");
	eval "flock(TMP,2);";
	print TMP "$in{no}<>$in{sub}<>$date<>$in{name}<>$in{url}<>$host<>$in{comment}<>$ext<>$w<>$h<>0<>$pwd<>";
	close(TMP);
	
	chmod(0666,"$cf{logdir}/$num.cgi");

	# クッキー格納
	set_cookie($in{name},$in{url}) if ($in{cookie});

	# メール通知
	my $clip = "$num$ext" if ($ext);
	mail_to_restmp($date,$sub,$clip) if ($cf{res_mailing} == 1);

	# 完了画面
	message("コメントを受理しました。<br />管理者の承認後に表示されます。");
}

#-------------------------------------------------
#  TOPレスカウント更新(親記事用INDEX)
#-------------------------------------------------
sub top_rescnt {

	# コメントログ
	my (%res, @data);
	open(DB,"$cf{reslog}");
	while( my $db = <DB> ) {
		my ($no,$ent,$sub,$date,$name,$url,$host,$com,$ex,$w,$h,$chk,$pw) = split(/<>/, $db);
		$res{$ent}++;
	}
	close(DB);

	# 記事ログ
	open(DB,"$cf{datadir}/log.cgi");
	while( my $db = <DB> ) {
		chomp($db);
		my ($no,$date,$name) = split(/<>/, $db);

		if (!defined($res{$no})) { $res{$no} = 0; }
		push(@data,"$no<>$date<>$name<>$res{$no}<>\n");

	}
	close(DB);

	# index更新
	open(DAT,"+< $cf{rescnt}");
	eval "flock(DAT, 2);";
	seek(DAT, 0, 0);
	print DAT @data;
	truncate(DAT, tell(DAT));
	close(DAT);
}

#-----------------------------------------------------------
#  コメント返信削除
#-----------------------------------------------------------
sub com_del {
	# 入力チェック
	$in{no} =~ s/\D//g;
	if ($in{no} eq '' || $in{pwd} eq '') {
		error("削除Noまたは削除キーが入力モレです");
	}

	my ($flg,$crypt,$ext,@log);
	open(DAT,"+< $cf{reslog}") or error("open err: $cf{reslog}");
	eval "flock(DAT, 2);";
	while (<DAT>) {
		my ($no,$ent,$sub,$date,$name,$url,$host,$com,$ex,$w,$h,$chk,$pw) = split(/<>/);

		if ($in{no} == $no) {
			$flg++;
			$crypt = $pw;
			$ext = $ex;
			next;
		}
		push(@log,$_);
	}

	if (!$flg || $crypt eq '') {
		close(DAT);
		error("削除キーが設定されていないか、該当記事が見当たりません");
	}

	# 削除キーを照合
	if (decrypt($in{pwd}, $crypt) != 1) {
		close(DAT);
		error("認証できません");
	}

	# 返信ログを更新
	seek(DAT, 0, 0);
	print DAT @log;
	truncate(DAT, tell(DAT));
	close(DAT);

	# 添付削除
	if ($ext) {
		unlink("$cf{r_upldir}/$in{no}$ext");
		unlink("$cf{r_upldir}/$in{no}-s$ext") if (-f "$cf{r_upldir}/$in{no}-s$ext");
	}

	# 連番修正
	open(DAT,"+< $cf{resnum}");
	eval "flock(DAT, 2);";
	my $num = <DAT> - 1;
	seek(DAT, 0, 0);
	print DAT $num;
	truncate(DAT, tell(DAT));
	close(DAT);

	# 親記事用INDEX
	top_rescnt();

	message("コメントを削除しました");
}

#-----------------------------------------------------------
#  コメント返信修正
#-----------------------------------------------------------
sub com_edit {

	# 入力チェック
	$in{no} =~ s/\D//g;
	if ($in{no} eq '' || $in{pwd} eq '') {
		&error("コメントNo又は暗証キーが入力されていません");
	}

	# 実行
	if ($in{job} eq "edit") {

	# 日本語チェック
	if ($cf{jp_word_chk}) {
		if ($in{comment} !~ /(?:[\xC0-\xDF][\x80-\xBF]|[\xE0-\xEF][\x80-\xBF]{2}|[\xF0-\xF7][\x80-\xBF]{3})/) {
			error("本文に日本語が含まれていません");
		}
	}
	
	# 禁止ワードチェック
	my $flg;
	for ( split(/\s+/,$cf{bad_word}) ) {
		if (index($in{comment},$_) >= 0) {
			$flg++;
			last;
		}
	}
	if ($flg) { error('コメントに禁止ワードが含まれているため受付できません!'); }
	
	# ホストチェック
	my ($addr,$host) = get_host();
	my $flg;
	for ( split(/\s+/,$cf{bad_host}) ) {
		if (index($host,$_) >= 0 or index($addr,$_) >= 0) {
			$flg++;
			last;
		}
	}
	if ($flg) { error('コメントは受付できません!!'); }

	# 不要改行カット
	$in{name} =~ s/<br>//g;
	$in{pwd}  =~ s/<br>//g;
	$in{captcha} =~ s/<br>//g;
	$in{comment} =~ s/(<br>)+$//g;

	# 入力値調整
	if ($in{url} eq 'http://') { $in{url} = ''; }

	# フォーム内容をチェック
	my $err;
	if ($in{name} eq "") { $err .= "名前が入力されていません<br>"; }
	if ($in{comment} eq "") { $err .= "コメントが入力されていません<br>"; }
	if ($in{url} ne '' && $in{url} !~ /^https?:\/\/[\w-.!~*'();\/?:\@&=+\$,%#]+$/) {
		$err .= "参照先URLの入力内容が不正です<br>";
	}
	if ($err) { error($err); }

		my ($flg, $ext2,$w2,$h2,@data);
		open(DAT,"+< $cf{reslog}") || &error("Open Error: $cf{reslog}");
		eval "flock(DAT, 2);";
		while (<DAT>) {
			my ($no,$ent,$sub,$date,$name,$url,$host,$com,$ex,$w,$h,$chk,$pw) = split(/<>/);

			if ($in{no} == $no) {
			$flg = 1;
			# アップロード
			if ($in{upfile}) {
				($ext2,$w2,$h2) = res_upload($in{no});
			}

			# 画像削除
			if ($in{imgdel}) {
				unlink("$cf{r_upldir}/$in{no}$ex");
				$ex = $w = $h = '';
			}

			# 添付アップロードの場合
			if ($ext2) {
				# 拡張子変更の場合旧ファイル削除
				if ($ex && $ext2 ne $ex) {
					unlink("$cf{r_upldir}/$in{no}$ex");
				}
				# 新添付ファイル情報
				($ex,$w,$h) = ($ext2,$w2,$h2);
			}
				$_ = "$no<>$ent<>$sub<>$date<>$in{name}<>$in{url}<>$host<>$in{comment}<>$ex<>$w<>$h<>$chk<>$pw<>\n";
			}
			push(@data,$_);
		}

		if ($flg != 1) {
			close(DAT);
			error("不正な処理です");
		}

		# 更新
		seek(DAT, 0, 0);
		print DAT @data;
		truncate(DAT, tell(DAT));
		close(DAT);

		# 完了メッセージ
		message("コメントの修正を完了しました");
	}

	my ($flg, @edit);
	open(IN,"$cf{reslog}") || &error("Open Error: $cf{reslog}");
	while (<IN>) {
		my ($no,$ent,$sub,$date,$name,$url,$host,$com,$ex,$w,$h,$chk,$pw) = split(/<>/);

		if ($in{no} == $no) {

			# 暗証キーなし
			if ($pw eq '') {
				$flg = 2;
			} elsif

			# 暗証キー一致
			(decrypt($in{pwd}, $pw) == 1) {
				$flg = 1;
				@edit = ($no,$ent,$sub,$date,$name,$url,$host,$com,$ex,$w,$h,$chk,$pw);

			# 暗証キー不一致
			} else {
				$flg = 3;
			}
			last;
		}
	}
	close(IN);

	if (!$flg) {
		close(DAT);
		error("該当コメントが見当たりません");
	} elsif ($flg == 2) {
		close(DAT);
		error("このコメントには暗証キーが設定されていません");
	} elsif ($flg == 3) {
		close(DAT);
		error("暗証キーが違います");
	}

	# 改行復元
	my ($no,$ent,$sub,$date,$name,$url,$host,$com,$ex,$w,$h,$chk,$pw) = split(/<>/,);
	$com =~ s|<br( /)?>|\n|g;
	$url ||= 'http://';

	# 画像認証作成
	my ($str_plain,$str_crypt,$regikey,$JavaScript);
	if ($cf{use_captcha} > 0) {
		require $cf{captcha_pl};
		($str_plain, $str_crypt, $regikey, $JavaScript) = cap::make( $cf{captcha_key}, $cf{cap_len} );
		$JavaScript=&mkJavaScript($regikey,'myFORM') if($cf{cap_len} == 3);
	}

	# 添付ファイル
	my $attach;
	if ($ex) {
		$attach .= qq|&nbsp; [<a href="$cf{r_upldir}/$no$ex" target="_blank">添付</a>]\n|;
		$attach .= qq|<input type="checkbox" name="imgdel" value="1">削除\n|;
	}

	# 修正フォーム
	open(IN,"$cf{tmpldir}/res_edit.html") or &error("open err: res_edit.html");
	print "Content-type: text/html; charset=utf-8\n\n";
	while(<IN>) {
                s/!str_crypt!/$str_crypt/g;
		s/!([a-z]+_cgi)!/$cf{$1}/g;
		s/!bbs_title!/$cf{bbs_title}/g;
		s|!bbs_css!|$cf{cmnurl}/clip.css|g;
		s|!mob_css!|$cf{cmnurl}/clip-mobile.css|g;
		s|!bbs_js!|$cf{cmnurl}/bbs.js|g;
		s/!num!/$no/g;
		s/!pwd!/$pw/g;
		s/!name!/$name/g;
		s/!sub!/$sub/g;
		s/!comment!/$com/g;
		s/!url!/$url/g;
		s/<!-- attach -->/$attach/g;
		print;
	}
	close(IN);

	# フッタ
	footer();

	exit;
}

#-----------------------------------------------------------
#  コメント返信添付アップロード
#-----------------------------------------------------------
sub res_upload {
	my $no = shift;

	# サムネイル機能
	require './lib/thumb.pl' if ($cf{thumbnail});

	# 拡張子取得
	my $ext = $cgi->param_filename('upfile') =~ /(\.[^\\\/:\.]+)$/ ? $1 : error("拡張子不明");
	$ext =~ tr/A-Z/a-z/;
	if ($ext eq '.jpeg') { $ext = '.jpg'; }

	# ファイルチェック
	check_upl($ext);

	# 添付ファイル定義
	my $upfile = "$cf{r_upldir}/$no$ext";

	# アップロード書き込み
	open(UP,"+> $upfile") or error("up err: $upfile");
	binmode(UP);
	print UP $in{upfile};
	close(UP);

	chmod(0666, $upfile);

	# 画像サイズ取得
	my ($flg,$w,$h);
	if ($ext eq ".jpg") { ($w,$h) = j_size($upfile); $flg++; }
	elsif ($ext eq ".gif") { ($w,$h) = g_size($upfile); $flg++; }
	elsif ($ext eq ".png") { ($w,$h) = p_size($upfile); $flg++; }

	# サムネイル化
	if ($flg && $cf{thumbnail}) {
		($w,$h) = resize_res($w,$h);
		my $thumb = "$cf{r_upldir}/$no-s$ext";
		make_thumb($upfile,$thumb,$w,$h);
	}

	# 結果を返す
	return ($ext,$w,$h);
}

#-----------------------------------------------------------
#  JPEGサイズ認識
#-----------------------------------------------------------
sub j_size {
	my $jpg = shift;
	
	my ($h, $w, $t);
	open(IMG,"$jpg");
	binmode(IMG);
	read(IMG, $t, 2);
	while (1) {
		read(IMG, $t, 4);
		my ($m, $c, $l) = unpack("a a n", $t);
		
		if ($m ne "\xFF") {
			$w = $h = 0;
			last;
		} elsif ((ord($c) >= 0xC0) && (ord($c) <= 0xC3)) {
			read(IMG, $t, 5);
			($h, $w) = unpack("xnn", $t);
			last;
		} else {
			read(IMG, $t, ($l - 2));
		}
	}
	close(IMG);
	
	return ($w,$h);
}

#-----------------------------------------------------------
#  GIFサイズ認識
#-----------------------------------------------------------
sub g_size {
	my $gif = shift;
	
	my $data;
	open(IMG,"$gif");
	binmode(IMG);
	sysread(IMG, $data, 10);
	close(IMG);
	
	if ($data =~ /^GIF/) { $data = substr($data, -4); }
	my $w = unpack("v", substr($data, 0, 2));
	my $h = unpack("v", substr($data, 2, 2));
	
	return ($w,$h);
}

#-----------------------------------------------------------
#  PNGサイズ認識
#-----------------------------------------------------------
sub p_size {
	my $png = shift;
	
	my $data;
	open(IMG,"$png");
	binmode(IMG);
	read(IMG, $data, 24);
	close(IMG);
	
	my $w = unpack("N", substr($data, 16, 20));
	my $h = unpack("N", substr($data, 20, 24));
	
	return ($w,$h);
}

#-----------------------------------------------------------
#  コメント返信メール送信
#-----------------------------------------------------------
sub mail_to_res {
	my ($date,$sub,$clip) = @_;

	# メールタイトル定義
	require './lib/jacode.pl';
	my $msub = mime_unstructured_header("BBS: $sub");

	# コメント内の改行復元
	my $com = $in{comment};
	$com =~ s|<br( /)?>|\n|g;
	$com =~ s/&lt;/>/g;
	$com =~ s/&gt;/</g;
	$com =~ s/&quot;/"/g;
	$com =~ s/&amp;/&/g;
	$com =~ s/&#39;/'/g;

	# URL情報
	if ($in{url} eq 'http://') { $in{url} = ''; }

	# メール本文を定義
	my $mbody = <<EOM;
$cf{bbs_title}にコメントがありました。

投稿日時 : $date
対象記事 : $sub

お名前：$in{name}
URL   ：$in{url}
添付  ：$clip

$com
EOM

	my $body;
	for my $tmp ( split(/\n/,$mbody) ) {
		jcode::convert(\$tmp,'jis','utf8');
		$body .= "$tmp\n";
	}
	
	# メールアドレスがない場合は管理者メールに置き換え
	$in{email} ||= $cf{mailto_r};
	
	# sendmailコマンド
	my $scmd = "$cf{sendmail} -t -i";
	if ($cf{sendm_f}) {
		$scmd .= " -f $in{email}";
	}

	# 送信
	open(MAIL,"| $scmd") or error("送信失敗");
	print MAIL "To: $cf{mailto_r}\n";
	print MAIL "From: $in{email}\n";
	print MAIL "Subject: $msub\n";
	print MAIL "MIME-Version: 1.0\n";
	print MAIL "Content-type: text/plain; charset=ISO-2022-JP\n";
	print MAIL "Content-Transfer-Encoding: 7bit\n";
	print MAIL "X-Mailer: $cf{version}\n\n";
	print MAIL "$body\n";
	close(MAIL);
}

#-----------------------------------------------------------
#  コメント返信(管理者承認)メール送信
#-----------------------------------------------------------
sub mail_to_restmp {
	my ($date,$sub,$clip) = @_;

	# メールタイトル定義
	require './lib/jacode.pl';
	my $msub = mime_unstructured_header("BBS: $sub");

	# コメント内の改行復元
	my $com = $in{comment};
	$com =~ s|<br( /)?>|\n|g;
	$com =~ s/&lt;/>/g;
	$com =~ s/&gt;/</g;
	$com =~ s/&quot;/"/g;
	$com =~ s/&amp;/&/g;
	$com =~ s/&#39;/'/g;

	# URL情報
	if ($in{url} eq 'http://') { $in{url} = ''; }

	# メール本文を定義
	my $mbody = <<EOM;
$cf{bbs_title}にコメントの受信がありましたので、お知らせします。
管理画面にログインして承認の可否を行ってください。

投稿日時 : $date
対象記事 : $sub

お名前：$in{name}
URL   ：$in{url}
添付  ：$clip

$com
EOM

	my $body;
	for my $tmp ( split(/\n/,$mbody) ) {
		jcode::convert(\$tmp,'jis','utf8');
		$body .= "$tmp\n";
	}
	
	# メールアドレスがない場合は管理者メールに置き換え
	$in{email} ||= $cf{mailto_r};
	
	# sendmailコマンド
	my $scmd = "$cf{sendmail} -t -i";
	if ($cf{sendm_f}) {
		$scmd .= " -f $in{email}";
	}

	# 送信
	open(MAIL,"| $scmd") or error("送信失敗");
	print MAIL "To: $cf{mailto_r}\n";
	print MAIL "From: $in{email}\n";
	print MAIL "Subject: $msub\n";
	print MAIL "MIME-Version: 1.0\n";
	print MAIL "Content-type: text/plain; charset=ISO-2022-JP\n";
	print MAIL "Content-Transfer-Encoding: 7bit\n";
	print MAIL "X-Mailer: $cf{version}\n\n";
	print MAIL "$body\n";
	close(MAIL);
}

#-----------------------------------------------------------
#  禁止ワードチェック
#-----------------------------------------------------------
sub no_wd {
	my $flg;
	foreach ( split(/,/, $cf{no_wd}) ) {
		if (index("$in{name} $in{sub} $in{comment}", $_) >= 0) {
			$flg = 1;
			last;
		}
	}
	if ($flg) { error("禁止ワードが含まれています"); }
}

#-----------------------------------------------------------
#  日本語チェック
#-----------------------------------------------------------
sub jp_wd {
	if ($in{comment} !~ /(?:[\xC0-\xDF][\x80-\xBF]|[\xE0-\xEF][\x80-\xBF]{2}|[\xF0-\xF7][\x80-\xBF]{3})/x) {
		error("メッセージに日本語が含まれていません");
	}
}

#-----------------------------------------------------------
#  URL個数チェック
#-----------------------------------------------------------
sub urlnum {
	my $com = $in{comment};
	my ($num) = ($com =~ s|(https?://)|$1|ig);
	if ($num > $cf{urlnum}) {
		error("コメント中のURLアドレスは最大$cf{urlnum}個までです");
	}
}

#-----------------------------------------------------------
#  アクセス制限
#-----------------------------------------------------------
sub get_host {
	# IP&ホスト取得
	my $host = $ENV{REMOTE_HOST};
	my $addr = $ENV{REMOTE_ADDR};

	if ($cf{gethostbyaddr} && ($host eq "" || $host eq $addr)) {
		$host = gethostbyaddr(pack("C4", split(/\./, $addr)), 2);
	}

	# IPチェック
	my $flg;
	foreach ( split(/\s+/, $cf{deny_addr}) ) {
		s/\./\\\./g;
		s/\*/\.\*/g;

		if ($addr =~ /^$_/i) { $flg = 1; last; }
	}
	if ($flg) {
		&error("アクセスを許可されていません");

	# ホストチェック
	} elsif ($host) {

		foreach ( split(/\s+/, $cf{deny_host}) ) {
			s/\./\\\./g;
			s/\*/\.\*/g;

			if ($host =~ /$_$/i) { $flg = 1; last; }
		}
		if ($flg) {
			&error("アクセスを許可されていません");
		}
	}
	if ($host eq "") { $host = $addr; }
	return ($host,$addr);
}

#-----------------------------------------------------------
#  crypt暗号
#-----------------------------------------------------------
sub encrypt {
	my $in = shift;

	my @wd = ('a'..'z', 'A'..'Z', '0'..'9', '.', '/');
	srand;
	my $salt = $wd[int(rand(@wd))] . $wd[int(rand(@wd))];
	crypt($in, $salt) || crypt ($in, '$1$' . $salt);
}

#-----------------------------------------------------------
#  crypt照合
#-----------------------------------------------------------
sub decrypt {
	my ($in, $dec) = @_;

	my $salt = $dec =~ /^\$1\$(.*)\$/ ? $1 : substr($dec, 0, 2);
	if (crypt($in, $salt) eq $dec || crypt($in, '$1$' . $salt) eq $dec) {
		return 1;
	} else {
		return 0;
	}
}

#-----------------------------------------------------------
#  完了メッセージ
#-----------------------------------------------------------
sub message {
	my ($msg) = @_;

	open(IN,"$cf{tmpldir}/message.html") or error("open err: message.html");
	my $tmpl = join('', <IN>);
	close(IN);

	$tmpl =~ s/!bbs_cgi!/$cf{bbs_cgi}/g;
	$tmpl =~ s/!bbs_title!/$cf{bbs_title}/g;
	$tmpl =~ s/!message!/$msg/g;
	$tmpl =~ s|!bbs_css!|$cf{cmnurl}/clip.css|g;
	$tmpl =~ s|!mob_css!|$cf{cmnurl}/clip-mobile.css|g;

	print "Content-type: text/html; charset=utf-8\n\n";
	print $tmpl;
	exit;
}

#-----------------------------------------------------------
#  フッター
#-----------------------------------------------------------
sub footer {
	my $foot = shift;

	# 著作権表記（削除・改変禁止）
	my $copy = <<EOM;
<div id="footer">
	- <a href="http://www.kent-web.com/" target="_top">$cf{version}</a> -<br>
	++ <a href="http://www.transformsite.com/" target="_top">Edited by TRANSFORM</a> ++
</div>
EOM

	if ($foot =~ /(.+)(<\/body[^>]*>.*)/si) {
		print "$1$copy$2\n";
	} else {
		print "$foot$copy\n";
		print "</body></html>\n";
	}
	exit;
}

#-----------------------------------------------------------
#  クッキー発行
#-----------------------------------------------------------
sub set_cookie {
	my @data = @_;

	my ($sec,$min,$hour,$mday,$mon,$year,$wday,undef,undef) = gmtime(time + 60*24*60*60);
	my @mon  = qw|Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec|;
	my @week = qw|Sun Mon Tue Wed Thu Fri Sat|;

	# 時刻フォーマット
	my $gmt = sprintf("%s, %02d-%s-%04d %02d:%02d:%02d GMT",
				$week[$wday],$mday,$mon[$mon],$year+1900,$hour,$min,$sec);

	# URLエンコード
	my $cook;
	foreach (@data) {
		s/(\W)/sprintf("%%%02X", unpack("C", $1))/eg;
		$cook .= "$_<>";
	}

	print "Set-Cookie: $cf{cookie_id}=$cook; expires=$gmt\n";
}

#-----------------------------------------------------------
#  ファイルチェック
#-----------------------------------------------------------
sub check_upl {
	my ($ext) = @_;

	# mimeタイプ
	my $mime = $cgi->param_mime('upfile');

	my $flg;
	if ($cf{ok_gif}) {
		if ($mime =~ /^image\/gif$/i and $ext eq '.gif') { $flg++; }
	}
	if (!$flg and $cf{ok_jpeg}) {
		if ($mime =~ /^image\/p?jpe?g$/i and $ext eq '.jpg') { $flg++; }
	}
	if (!$flg and $cf{ok_png}) {
		if ($mime =~ /^image\/(x-)?png$/i and $ext eq '.png') { $flg++; }
	}
	if (!$flg and $cf{ok_pdf}) {
		if ($mime =~ /^application\/pdf$/i and $ext eq '.pdf') { $flg++; }
	}
	if (!$flg and $cf{ok_zip}) {
		if ($mime =~ /^application\/(x-)?zip(-compressed)?$/i and $ext eq '.zip') { $flg++; }
	}
	if (!$flg and $cf{ok_text}) {
		if ($mime =~ /^text\/plain$/i and $ext =~ /^\.te?xt$/) { $flg++; }
	}
	if (!$flg and $cf{ok_word}) {
		if ($mime =~ /^application\/(vnd\.)?ms-?word$/i and $ext eq '.doc') { $flg++; }
		elsif ($mime =~ /^application\/vnd\.openxmlformats-officedocument\.wordprocessingml\.document$/i and $ext eq '.docx') { $flg++; }
	}
	if (!$flg and $cf{ok_excel}) {
		if ($mime =~ /^application\/(vnd\.)?ms-?excel$/i and $ext eq '.xls') { $flg++; }
		elsif ($mime =~ /^application\/vnd\.openxmlformats-officedocument\.spreadsheetml\.sheet$/i and $ext eq '.xlsx') { $flg++; }
	}
	if (!$flg and $cf{ok_ppt}) {
		if ($mime =~ /^application\/(vnd\.)?ms-?powerpoint$/i and $ext eq '.ppt') { $flg++; }
		elsif ($mime =~ /^application\/vnd\.openxmlformats-officedocument\.presentationml\.presentation$/i and $ext eq '.pptx') { $flg++; }
	}
	if (!$flg and $cf{ok_mp3}) {
		if ($mime =~ /^audio\/mpeg$/i and $ext eq '.mp3') { $flg++; }
		elsif ($mime =~ /^audio\/mp3/i and $ext eq '.mp3') { $flg++; }
		elsif ($mime =~ /^audio\/mpg/i and $ext eq '.mp3') { $flg++; }
	}
	if (!$flg and $cf{ok_mp4}) {
		if ($mime =~ /^video\/mpeg$/i and $ext eq '.mp4') { $flg++; }
		elsif ($mime =~ /^video\/mp4/i and $ext eq '.mp4') { $flg++; }
		elsif ($mime =~ /^video\/mpg/i and $ext eq '.mp4') { $flg++; }
	}

	if (!$flg) { error('このファイルは取り扱いできません'); }
}

#-----------------------------------------------------------
#  エラー画面
#-----------------------------------------------------------
sub error {
	my $err = shift;
	
	open(IN,"$cf{tmpldir}/error.html") or die;
	my $tmpl = join('',<IN>);
	close(IN);
	
	$tmpl =~ s/!(bbs_title|homepage|cmnurl|[a-z]+_cgi)!/$cf{$1}/g;
	$tmpl =~ s/!error!/$err/g;
	$tmpl =~ s/!cmnurl!/$cf{cmnurl}/;
	
	print "Content-type: text/html; charset=utf-8\n\n";
	print $tmpl;

	# フッタ
	footer();

	exit;
}

#-----------------------------------------------------------
#  フォームデコード
#-----------------------------------------------------------
sub parse_resform {
	my $cgi = shift;

	my %in;
	foreach ( $cgi->param() ) {
		my $val = $cgi->param($_);

		if ($_ ne 'upfile') {
			# 無効化
			$val =~ s/&/&amp;/g;
			$val =~ s/</&lt;/g;
			$val =~ s/>/&gt;/g;
			$val =~ s/"/&quot;/g;
			$val =~ s/'/&#39;/g;

			# 改行変換
			$val =~ s|\r\n|<br />|g;
			$val =~ s|[\n\r]|<br />|g;
		}

		$in{$_} = $val;
	}
	return %in;
}

#-----------------------------------------------------------
#  mimeエンコード
#  [quote] http://www.din.or.jp/~ohzaki/perl.htm#JP_Base64
#-----------------------------------------------------------
sub mime_unstructured_header {
  my $oldheader = shift;
  jcode::convert(\$oldheader,'euc','utf8');
  my ($header,@words,@wordstmp,$i);
  my $crlf = $oldheader =~ /\n$/;
  $oldheader =~ s/\s+$//;
  @wordstmp = split /\s+/, $oldheader;
  for ($i = 0; $i < $#wordstmp; $i++) {
    if ($wordstmp[$i] !~ /^[\x21-\x7E]+$/ and
	$wordstmp[$i + 1] !~ /^[\x21-\x7E]+$/) {
      $wordstmp[$i + 1] = "$wordstmp[$i] $wordstmp[$i + 1]";
    } else {
      push(@words, $wordstmp[$i]);
    }
  }
  push(@words, $wordstmp[-1]);
  foreach my $word (@words) {
    if ($word =~ /^[\x21-\x7E]+$/) {
      $header =~ /(?:.*\n)*(.*)/;
      if (length($1) + length($word) > 76) {
	$header .= "\n $word";
      } else {
	$header .= $word;
      }
    } else {
      $header = add_encoded_word($word, $header);
    }
    $header =~ /(?:.*\n)*(.*)/;
    if (length($1) == 76) {
      $header .= "\n ";
    } else {
      $header .= ' ';
    }
  }
  $header =~ s/\n? $//mg;
  $crlf ? "$header\n" : $header;
}
sub add_encoded_word {
  my ($str, $line) = @_;
  my $result;
  my $ascii = '[\x00-\x7F]';
  my $twoBytes = '[\x8E\xA1-\xFE][\xA1-\xFE]';
  my $threeBytes = '\x8F[\xA1-\xFE][\xA1-\xFE]';
  while (length($str)) {
    my $target = $str;
    $str = '';
    if (length($line) + 22 +
	($target =~ /^(?:$twoBytes|$threeBytes)/o) * 8 > 76) {
      $line =~ s/[ \t\n\r]*$/\n/;
      $result .= $line;
      $line = ' ';
    }
    while (1) {
      my $encoded = '=?ISO-2022-JP?B?' .
      b64encode(jcode::jis($target,'euc','z')) . '?=';
      if (length($encoded) + length($line) > 76) {
	$target =~ s/($threeBytes|$twoBytes|$ascii)$//o;
	$str = $1 . $str;
      } else {
	$line .= $encoded;
	last;
      }
    }
  }
  $result . $line;
}
# [quote] http://www.tohoho-web.com/perl/encode.htm
sub b64encode {
    my $buf = shift;
    my ($mode,$tmp,$ret);
    my $b64 = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
                . "abcdefghijklmnopqrstuvwxyz"
                . "0123456789+/";
	
    $mode = length($buf) % 3;
    if ($mode == 1) { $buf .= "\0\0"; }
    if ($mode == 2) { $buf .= "\0"; }
    $buf =~ s/(...)/{
        $tmp = unpack("B*", $1);
        $tmp =~ s|(......)|substr($b64, ord(pack("B*", "00$1")), 1)|eg;
        $ret .= $tmp;
    }/eg;
    if ($mode == 1) { $ret =~ s/..$/==/; }
    if ($mode == 2) { $ret =~ s/.$/=/; }
    
    return $ret;
}

